CQ.Ext.ns("CQ.Smartling");

CQ.Ext.onReady(function () {

    var fields = [
        'configId',
        'configName',
        'configType',
        'generalProjectId',
        'generalUserId',
        'generalUserSecret',
        'callbackUrl',
        'retrievalType',
        'fileTypes',
        'sourcePath',
        'sourceLocalePath',
        'sourceLocale',
        'targetInfos',
        'serverUri',
        'login',
        'password'
    ];

    var configStore = new CQ.Ext.data.JsonStore({
        root: 'configurations',
        idProperty: 'configId',

        url: '/services/translation/smtsiteconfig',
        baseParams: {},

        autoSave: false,
        writer: new CQ.Ext.data.JsonWriter({
            encode: false,
            listful: true,
            writeAllFields: true
        }),
        isDirty: function () {
            if (this.getModifiedRecords().length > 0) {
                return true;
            }
            var dirty = false;
            this.each(function (record) {
                if (record.deleted) {
                    dirty = true;
                    return false;
                }
            });
            return dirty;
        },
        markForRemove: function (record) {
            if (record) {
                record.deleted = true;
            }
        },
        proxy: new CQ.Ext.data.HttpProxy({
            api: {
                read: { url: '/services/translation/smtsiteconfig', method: 'GET' },
                update: '/services/translation/smtsiteconfig',
                create: '/services/translation/smtsiteconfig',
                destroy: '/services/translation/smtsiteconfig?delete=true'
            },
            headers: {"Content-Type": "application/json; charset=utf-8"}
        }),

        saveAll: function () {
            // delete records marked for deletion
            this.each(function (record) {
                if (record.deleted) {
                    this.remove(record);
                }
            }, this);
            this.save();
        },
        // overwrite to handle simple response with only "success: true" instead of
        // returning all ids of the written records (replacing DataReader.realize)
        onCreateRecords: function (success, rs, data) {
            this.commitChanges();
            this.each(function (record) {
                record.phantom = false;
            }, this);
        },

        fields: fields
    });

    configStore.on("beforesave", function () {
        grid.body.mask("Saving...");
    });

    configStore.on("save", function () {
        this.reload();
        grid.body.unmask();
    });

    var editContentDialog = new CQ.Smartling.ContentMappingDialog({
        configStore: configStore
    });

    var editDigitalAssetDialog = new CQ.Smartling.DigitalAssetMappingDialog({
        configStore: configStore
    });

    var editDictionaryDialog = new CQ.Smartling.DictionaryMappingDialog({
        configStore: configStore
    });

    var cmCfg = {
        // specify any defaults for each column
        defaults: {
            width: 120,
            sortable: true,
            flex: 1
        },
        columns: [
            {
                id: 'configName',
                header: "Config Name",
                dataIndex: 'configName'
            },
            {
                id: 'configType',
                header: "Config Type",
                dataIndex: 'configType'
            },
            {
                header: "Project Id",
                dataIndex: 'generalProjectId',
                width: 250
            },
            {
                header: "Callback URL",
                dataIndex: 'callbackUrl',
                width: 350
            },
            {
                header: "Retrieval Type",
                dataIndex: 'retrievalType'
            },
            {
                header: "Source Path",
                dataIndex: 'sourcePath',
                width: 300
            },
            {
                header: "Source Locale",
                dataIndex: 'sourceLocale'
            }
        ]
    };

    function showMenu(grid, index, event) {
        event.stopEvent();
        var record = grid.getStore().getAt(index);
        new CQ.Ext.menu.Menu({
            items: [
                {
                    text: 'Edit',
                    handler: function () {
                        editItem(record);
                    }
                },
                {
                    text: 'Clone',
                    handler: function () {
                        var configType = record.get("configType");
                        if (configType === "CONTENT") {
                            editContentDialog.cloneTranslations(record);
                        } else if (configType === "DIGITAL_ASSET") {
                            editDigitalAssetDialog.cloneTranslations(record);
                        } else if (configType === "DICTIONARY") {
                            editDictionaryDialog.cloneTranslations(record);
                        }
                    }
                },
                {
                    text: 'Delete',
                    handler: function () {
                        grid.getStore().markForRemove(record);
                        grid.getStore().saveAll();
                    }
                }
            ]
        }).showAt(event.xy);
    }

    function editItem(record) {
        var configType = record.get("configType");
        if (configType === "CONTENT") {
            editContentDialog.editTranslations(record);
        } else if (configType === "DIGITAL_ASSET") {
            editDigitalAssetDialog.editTranslations(record);
        } else if (configType === "DICTIONARY") {
            editDictionaryDialog.editTranslations(record);
        }
    }

    var grid = new CQ.Ext.grid.EditorGridPanel({
        region: "center",
        title: 'Configuration',
        layout: 'fit',
        margins: '5 5 5 5',
        //cls: 'custom-dirty',
        autoExpandColumn: 'configName',
        columnLines: true,
        selModel: new CQ.Ext.grid.RowSelectionModel(),
        loadMask: true,
        store: configStore,
        cm: new CQ.Ext.grid.ColumnModel(cmCfg),
        listeners: {
            'rowcontextmenu': function (grid, index, event) {
                showMenu(grid, index, event);
            },
            'rowdblclick': function (grid, index, event) {
                event.stopEvent();
                var record = grid.getStore().getAt(index);
                editItem(record);
            }
        }
    });

    var viewport = new CQ.Ext.Viewport({
        layout: "border",
        border: true,
        defaults: {
            border: true,
            margins: '5 5 5 5'
        },
        items: [
            {
                region: "center",
                layout: "fit",
                items: {
                    layout: "border",
                    title: "Smartling: Configurations",
                    border: false,
                    tbar: [
                        {
                            text: "Add Content Config",
                            iconCls: "add-icon",
                            handler: function () {
                                grid.stopEditing();
                                editContentDialog.addTranslations();
                            }
                        },
                        {
                            text: "Add Digital Asset Config",
                            iconCls: "add-icon",
                            handler: function () {
                                grid.stopEditing();
                                editDigitalAssetDialog.addTranslations();
                            }
                        },
                        {
                            text: "Add Dictionary Config",
                            iconCls: "add-icon",
                            handler: function () {
                                grid.stopEditing();
                                editDictionaryDialog.addTranslations();
                            }
                        },
                        {
                            text: "Reset & Refresh",
                            iconCls: "refresh-icon",
                            handler: function () {
                                configStore.reload();
                            }
                        }
                    ],
                    items: [
                        grid
                    ]
                }
            }
        ]
    });

    CQ.Smartling.load = function () {
        configStore.reload();
    };
});