CQ.Ext.ns('SMARTLING.ui');

SMARTLING.ui.AdvancedNodeTranslator = CQ.Ext.extend(CQ.Ext.Viewport, {

    constructor: function(config) {
        var nodeTranslator = this;

        var tree = new CQ.Ext.tree.TreePanel({
            'region': 'west',
            'margins': '5 0 5 5',
            'width': CQ.themes.wcm.SiteAdmin.TREE_WIDTH,
            'autoScroll': true,
            'containerScroll': true,
            'collapsible': true,
            'collapseMode': 'mini',
            'hideCollapseTool': true,
            'animate': true,
            'split': true,
            'loader': new CQ.Ext.tree.TreeLoader({
                'requestMethod': 'GET',
                'dataUrl': '/.tree.smartling.json',
                'baseParams': {
                    '_charset_': 'utf-8'
                },
                'baseAttrs': {
                    'singleClickExpand': true
                },
                'listeners': {
                    'beforeload': function (loader, node) {
                        var path = node.getPath() === '/' ? node.getPath() : node.getPath().substring(1);
                        loader.dataUrl = path + '.tree.smartling.json';
                    }
                }
            }),
            'root': new CQ.Ext.tree.AsyncTreeNode({
                'text': '/',
                'expanded': true
            }),
            'listeners': {
                'selectionchange': function (tree, path) {
                    var node = tree.selModel.getSelectedNode();

                    translateButton.setDisabled(!node.attributes.translatable);

                    propertiesGrid.store.proxy.setUrl(path.substring(1) + '.strings.smartling.json');
                    propertiesGrid.store.load({});
                }
            }
        });

        function newSubmissionHandler(bulk) {
            return function () {
                var node = tree.selModel.getSelectedNode();
                var config = {}
                config.paths = [node.getPath().replace('//', '/')];
                config.bulk = bulk;
                var dialog = new SMARTLING.ui.dialog.SendForTranslationDialog(config);
                dialog.show();
            }
        }

        var translateButton = new CQ.Ext.SplitButton({
            'disabled': true,
            'text': 'Translate',
            'menu': new CQ.Ext.menu.Menu({
                'items': [
                    {'text': 'Bulk translation', 'handler': newSubmissionHandler(true)}
                ]
            }),
            'handler': newSubmissionHandler(false)
        });

        var propertiesGrid = new CQ.Ext.grid.GridPanel({
            'region': 'center',
            'margins': '5 5 5 0',
            'store': new CQ.Ext.data.JsonStore({
                'url': '/',
                'fields': ['text']
                ,'listeners': {
                    'beforeload': function () {
                        nodeTranslator._mask();
                    },
                    'load': function() {
                        nodeTranslator._unmask();
                    },
                    'exception': function() {
                        nodeTranslator._unmask();
                    }
                }
            }),
            'cm': new CQ.Ext.grid.ColumnModel([
                {header: 'Text', dataIndex: 'text'}
            ]),
            'hideHeaders': true,
            'sm': new CQ.Ext.grid.RowSelectionModel({ 'singleSelect': true }),
            'viewConfig': new CQ.Ext.grid.GridView({ 'forceFit': true }),
            'tbar': [ translateButton ]
        });

        SMARTLING.ui.AdvancedNodeTranslator.superclass.constructor.call(this, {
            'layout': 'border',
            'items': [
                {
                    xtype: 'panel',
                    'id': 'smartling-advanced-translator',
                    'layout': 'border',
                    region: 'center',
                    'border': false,
                    'items': [
                        tree, propertiesGrid, new SMARTLING.ui.common.PageHeader()
                    ]
                }
            ]
        });
    },

    /**
     * Masks the main panel for loading.
     */
    _mask: function () {
        if (!this.loadMask) {
            this.loadMask = new CQ.Ext.LoadMask('CQ', {
                'msg': CQ.I18n.getMessage('Processing...')
            });
        }
        this.loadMask.show();
    },

    /**
     * Unmasks the main panel after loading.
     */
    _unmask: function() {
        if (this.loadMask) {
            this.loadMask.hide();
        }
    }
});
CQ.Ext.reg('smartling-node-translator', SMARTLING.ui.AdvancedNodeTranslator);