CQ.Ext.ns('SMARTLING.ui');

SMARTLING.ui.tcc = SMARTLING.ui.tcc || {};

SMARTLING.ui.tcc.Grid = CQ.Ext.extend(CQ.Ext.grid.GridPanel, {
    selected: [],
    toolBar: {},
    selectedPath: "/",
    actions: {},


    constructor: function (config) {
        var grid = this;

        for (var actionName in config.actions) {
            var action = config.actions[actionName];
            if (action && action.action) {
                grid.actions[actionName] = new SMARTLING.ui.common.TranslationAction(action, grid);
            }
        }

        var actionsArr = [];
        for (var actionName in grid.actions) {
            actionsArr.push(grid.actions[actionName]);
        }


        grid.toolBar = new SMARTLING.ui.tcc.ToolBar(function (dataType) {
            grid.reloadGrid(dataType)
        }, actionsArr);

        var contextMenu = new CQ.Ext.menu.Menu({
            items: actionsArr
        });

        SMARTLING.ui.tcc.Grid.superclass.constructor.call(grid, {
            region: 'center',
            store: new CQ.Ext.data.JsonStore({
                url: '/',
                fields: [
                    'name',
                    'title',
                    'targetTitle',
                    'lastModified',
                    'lastModifiedBy',
                    'lastReplicated',
                    'lastTargetReplicated',
                    'state',
                    'timeSubmission',
                    'timeApply',
                    'submitter',
                    'isUnderConfig',
                    'sourcePath',
                    'targetPath',
                    'dictionaryPath',
                    'nodePath',
                    'configType',
                    'fileUri',
                    'locale'
                ],
                listeners: {
                    beforeload: function (store, params) {
                        var s = grid.selectedPath === '/'? '/' : grid.selectedPath.substring(1);
                        s = s + '.strings.tcc.json' + (grid.toolBar.getCurrentLocale() ? '?locale=' + grid.toolBar.getCurrentLocale() : '');
                        store.proxy.setUrl(s);
                    }
                }
            }),
            hideHeaders: false,
            viewConfig: new CQ.Ext.grid.GridView({forceFit: true}),
            tbar: grid.toolBar.getToolbar(),
            cm: new CQ.Ext.grid.ColumnModel([]),
            listeners: {
                'rowdblclick': function (grid, rowIndex) {
                    var data = grid.store.getAt(rowIndex).data;
                    if (data.targetPath === null || data.configType !== 'CONTENT') {
                        CQ.Ext.Msg.show({
                            title: 'Warning',
                            msg: 'Preview mode is not available',
                            icon: CQ.Ext.Msg.INFO,
                            buttons: CQ.Ext.Msg.OK
                        });
                    }
                    else {
                        window.open('/apps/smartling/preview.html?source=' + data.sourcePath + '&target=' + data.targetPath);
                    }
                },
                'rowcontextmenu': function (grid, rowIndex, e) {
                    e.stopEvent();
                    contextMenu.showAt(e.getXY());
                    return false;
                }
            },
            sm: new CQ.Ext.grid.RowSelectionModel({
                singleSelect: false,
                listeners: {
                    rowselect: function (selectionModel, rowIndex, dataModel) {
                        var data = dataModel.data;
                        grid.selected.push(data[data.configType === 'CONTENT' ? 'name' : 'sourcePath']);
                        grid._setEnabled(grid.actions.translate, true);
                        if (data.configType === 'CONTENT') {
                            grid._setEnabled(grid.actions.translateWithChildren, true);
                        }
                        if (data['nodePath']) {
                            grid._setEnabled(grid.actions.download, true);
                            grid._setEnabled(grid.actions.cancel, true);
                            if (data.configType === 'CONTENT') {
                                grid._setEnabled(grid.actions.preview, true);
                            }
                        }
                    },
                    rowdeselect: function (selectionModel, rowIndex, dataModel) {
                        grid.selected.remove(dataModel.data[dataModel.data.configType === 'CONTENT' ? 'name' : 'sourcePath']);
                        for (var actionName in grid.actions) {
                            grid._setEnabled(grid.actions[actionName], false);
                        }
                    }
                }
            })

        });
    },

    translate: function (withChildren) {
        this.performRequest(withChildren, "translate");
    },

    download: function (withChildren) {
        this.performRequest(withChildren, "download");
    },

    reloadGrid: function (type) {
        if (type != null) {
            this.dataType = type;
        }
        if (this.dataType === 'CONTENT') {
            this.reconfigure(this.store, new CQ.Ext.grid.ColumnModel([
                {header: 'Name', dataIndex: 'name'},
                {header: 'Title', dataIndex: 'title'},
                {header: 'Target Title', dataIndex: 'targetTitle'},
                {header: 'Last Modified', dataIndex: 'lastModified'},
                {header: 'Last Modified By', dataIndex: 'lastModifiedBy'},
                {header: 'Last Replicated', dataIndex: 'lastReplicated'},
                {header: 'Last Target Replicated', dataIndex: 'lastTargetReplicated'},
                {header: 'Status', dataIndex: 'state'},
                {header: 'Time Submission', dataIndex: 'timeSubmission'},
                {header: 'Time Apply', dataIndex: 'timeApply'},
                {header: 'Submitter', dataIndex: 'submitter'}
            ]));
        } else if (this.dataType === 'DICTIONARY') {
            this.reconfigure(this.store, new CQ.Ext.grid.ColumnModel([
                {header: 'Source path', dataIndex: 'sourcePath'},
                {header: 'Target path', dataIndex: 'targetPath'},
                {header: 'Last Modified', dataIndex: 'lastModified'},
                {header: 'Last Modified By', dataIndex: 'lastModifiedBy'},
                {header: 'Last Replicated', dataIndex: 'lastReplicated'},
                {header: 'Last Target Replicated', dataIndex: 'lastTargetReplicated'},
                {header: 'Status', dataIndex: 'state'},
                {header: 'Time Submission', dataIndex: 'timeSubmission'},
                {header: 'Time Apply', dataIndex: 'timeApply'},
                {header: 'Submitter', dataIndex: 'submitter'}
            ]));
        } else {
            this.reconfigure(this.store, new CQ.Ext.grid.ColumnModel([]));
        }
        this.selected = [];
        this.store.load({});

    },

    setSelectedPath: function (selectedPath) {
        this.selectedPath = selectedPath;
    },

    setLangs: function (langs) {
        for (var actionName in this.actions) {
            this._setEnabled(this.actions[actionName], false);
        }
        if ( Boolean(langs && langs.length)) {
            this.toolBar.localeButton.setDisabled(false);
            this.toolBar.setLangs(langs);
        } else {
            this.toolBar.localeButton.setText('No configuration');
            this.toolBar.localeButton.setDisabled(true);
            this.toolBar.selectedLocale = null;
            this.store.removeAll();
        }
    },

    _setEnabled: function (action, enable) {
        if (action) {
            if (enable) {
                action.enable();
            } else {
                action.disable();
            }
        }
    },

    /**
     * Masks the main panel for loading.
     */
    _mask: function () {
        if (!this.loadMask) {
            this.loadMask = new CQ.Ext.LoadMask('CQ', {
                msg: CQ.I18n.getMessage('Processing...')
            });
        }
        this.loadMask.show();
    },

    /**
     * Unmasks the main panel after loading.
     */
    _unmask: function () {
        if (this.loadMask) {
            this.loadMask.hide();
        }
    }


});