CQ.Ext.ns('SMARTLING.ui');

SMARTLING.ui.PropertyMappingWidget = CQ.Ext.extend(CQ.form.CompositeField, {

    hiddenField: null,
    propertyLabel: null,
    property: null,
    parserLabel: null,
    parser: null,

    constructor: function(config) {
        config = config || { };
        var defaults = {
            "border": false,
            "layout": "table",
            "columns": 4
        };

        config = CQ.Util.applyDefaults(config, defaults);
        SMARTLING.ui.PropertyMappingWidget.superclass.constructor.call(this, config);
    },

    // overriding CQ.Ext.Component#initComponent
    initComponent: function() {
        SMARTLING.ui.PropertyMappingWidget.superclass.initComponent.call(this);

        this.hiddenField = new CQ.Ext.form.Hidden({
            name: this.name
        });
        this.add(this.hiddenField);

        this.propertyLabel = new CQ.Ext.form.Label({
            name: "propertyLabel",
            text: "Property",
            width: 100,
            padding: 5,
            style: "font: normal 12px tahoma, arial, helvetica, sans-serif; width: 100px; padding: 5px"
        });
        this.add(this.propertyLabel);

        this.property = new CQ.Ext.form.TextField({
            name: "property",
            width: 190,
            padding: 5,
            listeners: {
                change: {
                    scope:this,
                    fn:this.updateHidden
                }
            }
        });
        this.add(this.property);

        this.parserLabel = new CQ.Ext.form.Label({
            name: "parserLabel",
            text: "Parser",
            style: "font: normal 12px tahoma, arial, helvetica, sans-serif; width: 100px; padding: 5px"
        });
        this.add(this.parserLabel);

        this.parser = new CQ.form.Selection({
            type: "select",
            name: "parser",
            width: 190,
            padding: 5,
            listeners: {
                selectionchanged: {
                    scope:this,
                    fn: this.updateHidden
                }
            },
            options: "/services/translation/parsertypes",
            optionsRoot: "parserTypes",
            optionsTextField: "label",
            optionsValueField: "value",
            value: "html"
        });
        this.add(this.parser);
    },

    // overriding CQ.form.CompositeField#processPath
    processPath: function(path) {
        this.parser.processPath(path);
    },

    // overriding CQ.form.CompositeField#processRecord
    processRecord: function(record, path) {
        this.parser.processRecord(record, path);
    },

    // overriding CQ.form.CompositeField#setValue
    setValue: function(value) {
        if (value && value.indexOf("parser") !== -1) {
            var map = JSON.parse(value);
            this.property.setValue(map.property);
            this.parser.setValue(map.parserType);
        } else {
            this.property.setValue(value);
            this.parser.setValue("html");
        }

        this.hiddenField.setValue(value);
    },

    // overriding CQ.form.CompositeField#getValue
    getValue: function() {
        return this.getRawValue();
    },

    // overriding CQ.form.CompositeField#getRawValue
    getRawValue: function() {
        if (!this.property || !this.parser) {
            return null;
        }
        return JSON.stringify({property: this.property.getValue(), parserType: this.parser.getValue(), parserDescription: this.parser.getRawValue()});
    },

    // private
    updateHidden: function() {
        this.hiddenField.setValue(this.getValue());
    }

});

// register xtype
CQ.Ext.reg('property-mapping-widget', SMARTLING.ui.PropertyMappingWidget);